/*******************************************************************************
 * Copyright 2015 - 2018 fmCreation
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/
package de.frankmuenster.jameica.qrscanner;

import org.apache.commons.lang.StringUtils;

import de.willuhn.jameica.system.Application;
import de.willuhn.util.I18N;

/**
 * Settings Klasse fr den jameica.qrscanner Basiert auf den Settings-Klasse aus dem Beispiel-Plugin von Olaf Willuhn.
 *
 * @author Frank Mnster
 * @see <a href= "http://www.willuhn.de/products/jameica/dev.php">http://www.willuhn.de/products/jameica/dev.php</a>
 */
public final class Settings {

    /**
     * Anzeige Text fr keine IP Kamera Untersttung
     */
    public static final String NO_IP_CAM = "Keine";  //$NON-NLS-1$

    /**
     * Standard IP Kamera Name
     */
    public static final String STANDRD_IP_CAM = "Standard IP Kamera";  //$NON-NLS-1$

    /**
     * System settings von Jameica
     */
    private static de.willuhn.jameica.system.Settings jameicaSettings = Application.getPluginLoader()
            .getPlugin(QrCodeScanner.class).getResources().getSettings();

    /**
     * I18N local Variable
     */
    private static I18N i18n;

    /**
     * Is Hibiscus plugin also installed
     */
    private static Boolean hibiscusInstalled;

    /**
     * Standardkonstruktor
     */
    private Settings() {
        // Absichtlich leer
    }

    /**
     * Small helper function to get the translator.
     *
     * @return translator.
     */
    public static I18N getI18n() {
        if (Settings.i18n == null) {
            Settings.i18n = Application.getPluginLoader().getPlugin(QrCodeScanner.class).getResources().getI18N();
        }
        return Settings.i18n;
    }

    /**
     * berprft, ob das Hibiscus -Plugin installiert ist.
     *
     * @return isHibiscusInstalled <b>true</b>, wenn Hibiscus installiert ist, <b>false</b> sonst
     */
    public static Boolean isHibiscusInstalled() {
        if (Settings.hibiscusInstalled == null) {
            Settings.hibiscusInstalled = Application.getPluginLoader().isInstalled("de.willuhn.jameica.hbci.HBCI"); //$NON-NLS-1$
        }
        return Settings.hibiscusInstalled;
    }

    /**
     * Liefert die zu verwendende IP Kamera.
     *
     * @return die zu verwendende IP Kamera
     */
    public static String getIpCam() {
        return Settings.jameicaSettings.getString("qrcode.ipCam", Settings.NO_IP_CAM); //$NON-NLS-1$
    }

    /**
     * Setzt, die zu verwendende IP Kamera.
     *
     * @param ipCam
     *            benutzte IP Kamera
     */
    public static void setIpCam(final String ipCam) {
        Settings.jameicaSettings.setAttribute("qrcode.ipCam", ipCam); //$NON-NLS-1$
    }

    /**
     * Liefert den IpCam Host.
     *
     * @return der IpCam Host.
     */
    public static String getIpCamHost() {
        return Settings.jameicaSettings.getString("qrcode.droidCamHost", StringUtils.EMPTY); //$NON-NLS-1$
    }

    /**
     * Setzt den IpCam Host.
     *
     * @param ipCamHost
     *            der IpCam Host.
     */
    public static void setIpCamHost(final String ipCamHost) {
        Settings.jameicaSettings.setAttribute("qrcode.droidCamHost", ipCamHost); //$NON-NLS-1$
    }

    /**
     * Liefert den gespeicherten IpCam Port
     *
     * @return den IpCam Port
     */
    public static Integer getIpCamPort() {
        return Settings.jameicaSettings.getInt("qrcode.droidCamPort", -1); //$NON-NLS-1$
    }

    /**
     * Setzt den IpCam Port.
     *
     * @param ipCamPort
     *            der IpCam port.
     */
    public static void setIpCamPort(final int ipCamPort) {
        Settings.jameicaSettings.setAttribute("qrcode.droidCamPort", ipCamPort); //$NON-NLS-1$
    }

    /**
     * Liefert den SEPA-Purpose-Code
     *
     * @return den SEPA-Purpose-Code
     */
    public static String getPursposeCode() {
        return Settings.jameicaSettings.getString("qrcode.pursposeCode", StringUtils.EMPTY); //$NON-NLS-1$
    }

    /**
     * Setzt den SEPA-Purpose-Code
     *
     * @param purposeCode
     *            den SEPA-Purpose-Code
     */
    public static void setPursposeCode(final String purposeCode) {
        Settings.jameicaSettings.setAttribute("qrcode.pursposeCode", purposeCode); //$NON-NLS-1$
    }

    /**
     * Liefert die default Konto ID
     * 
     * @return liefert die default Konto ID
     */
    public static String getDefaultAccountId() {
        return Settings.jameicaSettings.getString("qrcode.accountId", null); //$NON-NLS-1$
    }

    /**
     * Setzt die default Konto ID
     * 
     * @param accountId
     *            die default Konto ID
     */
    public static void setDefaultAccountId(final String accountId) {
        Settings.jameicaSettings.setAttribute("qrcode.accountId", accountId); //$NON-NLS-1$
    }
}
