/*******************************************************************************
 * Copyright 2015 - 2018 fmCreation
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/
package de.frankmuenster.jameica.qrscanner.cliurlhandler;

import org.apache.commons.lang.StringUtils;

import de.frankmuenster.jameica.qrcodehandler.QrCodeHandler;
import de.frankmuenster.jameica.qrcodehandler.QrCodeHandlerException;
import de.frankmuenster.jameica.qrcodehandler.QrCodeHandlerRegsistry;
import de.frankmuenster.jameica.qrscanner.Settings;
import de.willuhn.jameica.gui.AbstractView;
import de.willuhn.jameica.gui.GUI;
import de.willuhn.jameica.messaging.Message;
import de.willuhn.jameica.messaging.MessageConsumer;
import de.willuhn.jameica.messaging.SystemMessage;
import de.willuhn.jameica.system.Application;
import de.willuhn.logging.Logger;

/**
 * Bearbeitet 'QR-Codes' die als URL per Kommandozeile bergeben wurden.
 *
 * @author Frank Mnster
 */
public class CliUrlHandler implements MessageConsumer {

    /**
     * URL mit der Jameica gestartet wurde
     */
    private final transient String url;

    /**
     * Erstelle eine neu Instanz des CliUrlHandler Objektes. Leere Konstruktor wird von Jamecia MessageConsumer
     * gefordert.
     */
    public CliUrlHandler() {
        this.url = StringUtils.EMPTY;
    }

    /**
     * Erstelle eine neu Instanz des CliUrlHandler Objektes
     * 
     * @param url
     *            die URL die als Kommandozeilenparameter bearbeitet werden soll. Muss mit bank:// anfangen.
     */
    public CliUrlHandler(final String url) {
        this.url = url;
    }

    /**
     * @see de.willuhn.jameica.messaging.MessageConsumer#getExpectedMessageTypes()
     */
    @SuppressWarnings("rawtypes")
    @Override
    public Class[] getExpectedMessageTypes() {
        return new Class[] { SystemMessage.class };
    }

    /**
     * @see de.willuhn.jameica.messaging.MessageConsumer#handleMessage (de.willuhn.jameica.messaging.Message)
     */
    @SuppressWarnings("PMD.SignatureDeclareThrowsException")
    @Override
    public void handleMessage(final Message message) throws Exception {
        final SystemMessage sysMsg = (SystemMessage) message;
        if (sysMsg.getStatusCode() == SystemMessage.SYSTEM_STARTED) {
            Logger.debug("System gestartet. Nun kan SEPA-berweisung angelegt werden."); //$NON-NLS-1$
            final HandlerThread handlerThread = new HandlerThread();
            handlerThread.start();
        }
    }

    /**
     * @see de.willuhn.jameica.messaging.MessageConsumer#autoRegister()
     */
    @Override
    public boolean autoRegister() {
        return false;
    }

    private void unregister() {
        // Unregister
        Application.getMessagingFactory().unRegisterMessageConsumer(this);
    }

    /**
     * Private Klasse um im Hintergrund auf die StartView zu warten
     */
    private class HandlerThread extends Thread {

        /**
         * Standardkonstruktor
         */
        public HandlerThread() {
            super();
        }

        /**
         * Start the Thread
         */
        @Override
        public void run() {
            int waitLoopCount = 0;
            while (waitLoopCount < 20) {
                // warten bis StartView geladen. max 20 sec.
                final AbstractView currentView = GUI.getCurrentView();
                if (currentView != null) {
                    break;
                }
                waitLoopCount++;
                try {
                    Thread.sleep(1000);
                } catch (final InterruptedException e) {
                    // Nothing to do here
                }
            }
            final QrCodeHandler handler = QrCodeHandlerRegsistry.getQrCodeHandler(CliUrlHandler.this.url);
            if (handler == null) {
                Logger.info(Settings.getI18n().tr("No matching qr-code handler found"));
                final String niceText = "Fr den gefundenen QR-Code (siehe debug log) wurde \n"
                        + "kein passender Handler zur weiteren Verarbeitung\n"
                        + "gefunden.\n\nBitte berprfen Sie den QR-Code.";
                try {
                    Application.getCallback().notifyUser(niceText);
                } catch (final Exception e) { // NOPMD
                    Logger.error("Benutzer konnte nicht benachrichtigt werden.", e);
                }
            } else {
                try {
                    handler.handleQrCode(CliUrlHandler.this.url);
                } catch (final QrCodeHandlerException qre) {
                    Logger.error("QR-Code handler error: " + qre.getMessage());
                    final String niceText = "Der QR-Code konnte nicht vollstndig bearbeitet werden.\n\n"
                            + "Bitte berprfen Sie die System-Logs fr weitere Informationen.\n";
                    try {
                        Application.getCallback().notifyUser(niceText);
                    } catch (final Exception e) { // NOPMD
                        Logger.error("Benutzer konnte nicht benachrichtigt werden.", e);
                    }
                }
            }
            // Unregister Message Consumer
            CliUrlHandler.this.unregister();
        }
    }
}
