/*******************************************************************************
 * Copyright 2015 - 2018 fmCreation
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/

package de.frankmuenster.jameica.qrscanner.gui.action;

import de.frankmuenster.jameica.qrcodehandler.QrCodeHandler;
import de.frankmuenster.jameica.qrcodehandler.QrCodeHandlerException;
import de.frankmuenster.jameica.qrcodehandler.QrCodeHandlerRegsistry;
import de.frankmuenster.jameica.qrscanner.Settings;
import de.frankmuenster.jameica.qrscanner.gui.dialog.QrScannerDialog;
import de.willuhn.jameica.gui.Action;
import de.willuhn.jameica.system.Application;
import de.willuhn.jameica.system.OperationCanceledException;
import de.willuhn.logging.Logger;
import de.willuhn.util.ApplicationException;

/**
 * View und Kontrolleur zum Scannen und weiter bearbeiten eines QR-Codes.
 *
 * @author Frank Mnster
 */
public class QrScannerAction implements Action {

    /**
     * Standardkonstruktor
     */
    public QrScannerAction() {
	// Absichtlich leer
    }

    /**
     * ffnet einen QrScannerDialog und gibt das Ergebnis an einen passenden
     * QR-Code Handler weiter, falls mglich.
     */
    @Override
    public void handleAction(final Object context) throws ApplicationException {
	String qrText = null;
	// show dialog and start scanning
	try {
	    final QrScannerDialog dialog = new QrScannerDialog();
	    try {
		qrText = dialog.open();
	    } catch (final OperationCanceledException e) {
		Logger.info("Einlesen vom Benutzer abgebrochen");
	    } catch (final Exception e) { // NOPMD
		Logger.error("Dialogfenster konnte nicht geffnet werden.", e);
	    }
	    if (qrText == null) {
		Logger.debug("Kein QR String gefunden, oder Dialog mittels X geschlossen");
	    } else {
		Logger.debug("Scann Result:");
		Logger.debug(qrText);
		final QrCodeHandler handler = QrCodeHandlerRegsistry.getQrCodeHandler(qrText);
		if (handler == null) {
		    Logger.info(Settings.getI18n().tr("No matching qr-code handler found"));
		    final String niceText = "Fr den gefundenen Qr-Code (siehe debug log) wurde \n"
			    + "kein passender Handler zur weiteren Verabeitung\n"
			    + "gefunden.\n\nBitte berprfen Sie den Qr-Code.";

		    try {
			Application.getCallback().notifyUser(niceText);
		    } catch (final Exception e) { // NOPMD
			// Hier brauchen wir nichts zu tun!
			Logger.error("Benutzerbenachrichtigung fehlgeschlagen!", e);
		    }
		} else {
		    try {
			handler.handleQrCode(qrText);
		    } catch (final QrCodeHandlerException qre) {
			Logger.error("Qr-Code handler error: " + qre.getMessage());
			final String niceText = "Der QR-Code konnte nicht vollstndig bearbeitet werden.\n\n"
				+ "Bitte berprfen Sie die System-Logs fr weitere Informationen.\n";
			try {
			    Application.getCallback().notifyUser(niceText);
			} catch (final Exception e) { // NOPMD
			    // Hier brauchen wir nichts zu tun!
			    Logger.error("Benutzerbenachrichtigung fehlgeschlagen!", e);
			}
		    }
		}
	    }
	} catch (final OperationCanceledException oce) {
	    Logger.info("Einlesen des QR Codes vom Benutzer abgebrochen");
	    try {
		Thread.sleep(1000);
	    } catch (final InterruptedException e) {
	    }
	}
    }
}
