/*******************************************************************************
 * Copyright 2015 - 2018 fmCreation
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/
package de.frankmuenster.jameica.qrscanner.gui.controller;

import java.io.File;
import java.io.FileInputStream;

import de.frankmuenster.jameica.qrscanner.QrCodeScanner;
import de.frankmuenster.jameica.qrscanner.Settings;
import de.willuhn.io.FileFinder;
import de.willuhn.jameica.gui.AbstractControl;
import de.willuhn.jameica.gui.AbstractView;
import de.willuhn.jameica.gui.Part;
import de.willuhn.jameica.gui.parts.FormTextPart;
import de.willuhn.jameica.plugin.AbstractPlugin;
import de.willuhn.jameica.plugin.Manifest;
import de.willuhn.jameica.system.Application;
import de.willuhn.jameica.util.InfoReader;
import de.willuhn.logging.Logger;
import de.willuhn.util.I18N;

/**
 * Controller fuer den Lizenzinformationen Dialog Basiert auf dem
 * {@link de.willuhn.jameica.gui.internal.controller#LincenseControl}
 *
 * @author Frank Mnster
 */
public class LicenseControl extends AbstractControl {

    /**
     *
     */
    private static final String HTML_BR = "<br/>"; //$NON-NLS-1$

    /**
     * i18n bersetzungs-Objekt aus unseren Settings
     */
    private static I18N i18n = Settings.getI18n();

    /**
     * FormTextPart um dem Text schn formatieren zu knnen
     */
    private transient FormTextPart libList;

    /**
     * Standard Konstruktor liefert ein LincenceControl Objekt
     *
     * @param view
     *            die View, fr die dieser LicenceControl zustndig ist
     */
    public LicenseControl(final AbstractView view) {
        super(view);
    }

    /**
     * Liefert eine Liste mit allen direkt von jameica.qrscanner verwendeten Komponenten.
     *
     * @return Liste der Komponenten.
     */
    public Part getLibList() {
        if (this.libList == null) {
            final AbstractPlugin plugin = Application.getPluginLoader().getPlugin(QrCodeScanner.class);
            final StringBuilder buffer = new StringBuilder();
            Manifest manifest = null;
            try {
                manifest = plugin.getManifest();
            } catch (final Exception e) { // NOPMD
                Logger.error("unable to read info.xml from plugin jameica.qrscanner", e);
            }
            buffer.append("<form><p><span color=\"header\" font=\"header\">" + LicenseControl.i18n.tr("QR-Code Scanner")
                    + "</span></p>");
            if (manifest != null) {
                buffer.append("<p>");
                buffer.append(manifest.getDescription());
                buffer.append(LicenseControl.HTML_BR).append(manifest.getHomepage());
                buffer.append(LicenseControl.HTML_BR).append(manifest.getLicense());
                buffer.append("</p>");
            }
            final String path = plugin.getManifest().getPluginDir();
            final FileFinder finder = new FileFinder(new File(path + "/lib"));
            finder.matches(".*?info\\.xml$");
            final File[] infos = finder.findRecursive();
            for (int i = 0; i < infos.length; ++i) {
                if (!infos[i].isFile() || !infos[i].canRead()) {
                    Logger.warn("unable to read " + infos[i] + ", skipping");
                    continue;
                }
                try {
                    final InfoReader infoReader = new InfoReader(new FileInputStream(infos[i])); // NOPMD
                    buffer.append("<p><b>" + infoReader.getName() + "</b>");
                    buffer.append(LicenseControl.HTML_BR + infoReader.getDescription());
                    buffer.append(LicenseControl.HTML_BR + infoReader.getUrl());
                    buffer.append(LicenseControl.HTML_BR + infoReader.getLicense());
                    buffer.append("</p>");
                } catch (final Exception e) { // NOPMD
                    Logger.error("unable to parse " + infos[0], e);
                }
            }
            buffer.append("</form>");
            this.libList = new FormTextPart(buffer.toString());
        }
        return this.libList;
    }
}
