/*******************************************************************************
 * Copyright 2015 - 2018 fmCreation
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/
package de.frankmuenster.jameica.qrscanner.gui.controller;

import java.net.MalformedURLException;
import java.rmi.RemoteException;
import java.util.List;

import org.apache.commons.lang.StringUtils;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Listener;

import com.github.sarxos.webcam.Webcam;
import com.github.sarxos.webcam.ds.ipcam.IpCamDevice;
import com.github.sarxos.webcam.ds.ipcam.IpCamDeviceRegistry;
import com.github.sarxos.webcam.ds.ipcam.IpCamMode;

import de.frankmuenster.jameica.qrscanner.Settings;
import de.frankmuenster.jameica.qrscanner.WebcamWatchdog;
import de.frankmuenster.jameica.qrscanner.gui.views.SettingsView;
import de.frankmuenster.jameica.qrscanner.ipcam.DroidCamDevice;
import de.frankmuenster.jameica.qrscanner.ipcam.IphoneIpCameraCamDevice;
import de.frankmuenster.jameica.qrscanner.utils.I18N;
import de.frankmuenster.jameica.qrscanner.utils.Utils;
import de.frankmuenster.jameica.qrscanner.utils.Validators;
import de.willuhn.jameica.gui.AbstractControl;
import de.willuhn.jameica.gui.Action;
import de.willuhn.jameica.gui.GUI;
import de.willuhn.jameica.gui.input.IntegerInput;
import de.willuhn.jameica.gui.input.SelectInput;
import de.willuhn.jameica.gui.input.TextAreaInput;
import de.willuhn.jameica.gui.input.TextInput;
import de.willuhn.jameica.gui.parts.Button;
import de.willuhn.jameica.hbci.gui.filter.KontoFilter;
import de.willuhn.jameica.hbci.gui.input.KontoInput;
import de.willuhn.jameica.hbci.gui.input.PurposeCodeInput;
import de.willuhn.jameica.hbci.rmi.Konto;
import de.willuhn.jameica.hbci.synchronize.jobs.SynchronizeJobSepaUeberweisung;
import de.willuhn.logging.Logger;
import de.willuhn.util.ApplicationException;

/**
 * Kontrolleur fr die Settings View
 *
 * @author Frank Mnster
 */
@SuppressWarnings({ "PMD.ExcessiveImports" })
public class SettingsController extends AbstractControl {

    /**
     * Vorbelegung des SEPA-Purpose-Codes
     */
    private transient PurposeCodeInput purposeCode;

    /**
     * Abbuchungsungskonto der berweisung
     */
    private transient KontoInput defaultAccount;

    /**
     * Checkbox Eingabefeld es DroidCam ist
     */
    private transient SelectInput ipCam;

    /**
     * IPCam Host
     */
    private transient TextInput ipCamHost;

    /**
     * IPCam Port
     */
    private transient IntegerInput ipCamPort;

    /**
     * Anzeigebereich fr gefundene Webcams
     */
    private transient TextAreaInput discoverdWebcams;

    /**
     * Aktualisieren Button
     */
    private transient Button refreshBtn;

    /**
     * Erstellt eine neue Instanz des Kontrolleurs
     *
     * @param view
     */
    public SettingsController(final SettingsView view) {
        super(view);
    }

    /**
     * Liefert ein Texteingabefeld fr den SEPA-Purpose-Codes
     *
     * @return TextInput fr den SEPA-Purpose-Codes
     */
    public PurposeCodeInput getPurposeCode() {
        if (this.purposeCode == null) {
            this.purposeCode = new PurposeCodeInput(Settings.getPursposeCode());
        }
        return this.purposeCode;
    }

    /**
     * @return Ein CheckBoxInput zur Auswahl, ob die DroidCam App als Webcam benutzt werden soll.
     */
    public SelectInput getIpCam() {
        if (this.ipCam == null) {
            Logger.debug("IpCam: " + Settings.getIpCam()); //$NON-NLS-1$
            final String[] ipCams = { Settings.NO_IP_CAM, Settings.STANDRD_IP_CAM, DroidCamDevice.CAM_NAME,
                    IphoneIpCameraCamDevice.CAM_NAME };
            this.ipCam = new SelectInput(ipCams, Settings.getIpCam());
            this.ipCam.addListener(new Listener() {

                /**
                 * @see Listener#handleEvent(Event)
                 */
                @Override
                public void handleEvent(final Event event) {
                    // SettingsController.this.handleUseDroidCamEvent(event);
                }
            });
            this.ipCam.setComment(StringUtils.EMPTY);
        }
        return this.ipCam;
    }

    /**
     * Liefert ein Texteingabefeld f&uuml;r den DroidCam Host
     *
     * @return TextInput f&uuml;r DroidCam IP
     */
    public TextInput getIpCamHost() {
        if (this.ipCamHost == null) {
            this.ipCamHost = new TextInput(Settings.getIpCamHost(), 15, "192.178.0.200"); //$NON-NLS-1$ // NOPMD
        }
        return this.ipCamHost;
    }

    /**
     * Liefert ein DecimalInput f&uuml;r den DroidCam Port.
     *
     * @return TextInput f&uuml;r DroidCam Port
     */
    public IntegerInput getIpCamPort() {
        if (this.ipCamPort == null) {
            this.ipCamPort = new IntegerInput(Settings.getIpCamPort());
            this.ipCamPort.setComment(StringUtils.EMPTY);
        }
        return this.ipCamPort;
    }

    /**
     * Liefert ein Kontoauswahlfeld
     *
     * @return Kontoauswahlfeld
     */
    public KontoInput getDefaultAccount() {
        if (this.defaultAccount == null) {
            final Konto konto = Utils.getKonto(Settings.getDefaultAccountId());
            try {
                final KontoFilter ktoFilter = KontoFilter.createForeign(SynchronizeJobSepaUeberweisung.class);
                this.defaultAccount = new KontoInput(konto, ktoFilter);
            } catch (final RemoteException e) {
                Logger.error("Fehler beim Erstellen der KontoListe.", e); //$NON-NLS-1$
            }
        }
        return this.defaultAccount;
    }

    /**
     * Liefert ein Anzeigebereich fr gefundene Webcams.
     *
     * @return TextAreaInput zum Anzeigen gefundener Webcams.
     */
    public TextAreaInput getDiscoverdWebcams() {
        if (this.discoverdWebcams == null) {
            this.discoverdWebcams = new TextAreaInput(I18N.tr("SettingsController.noWebcamFound")); //$NON-NLS-1$
            this.discoverdWebcams.setHeight(130);
            this.discoverdWebcams.setComment(StringUtils.EMPTY);
        }
        final List<Webcam> webcams = Webcam.getWebcams();
        final StringBuffer webcamNames = new StringBuffer();
        for (final Webcam webcam : webcams) {
            webcamNames.append(webcam.getName()).append('\n');
        }
        this.discoverdWebcams.setValue(webcamNames.toString());
        return this.discoverdWebcams;
    }

    /**
     * Liefert einen Aktualisierung-Button
     *
     * @return einen Aktualisierung-Button
     */
    public Button getRefeshBtn() {
        if (this.refreshBtn == null) {
            this.refreshBtn = new Button(I18N.tr("SettingsController.Refresh"), new Action() { //$NON-NLS-1$

                /**
                 * Action#handleAction(Object)
                 */
                @Override
                public void handleAction(final Object context) throws ApplicationException {
                    // JmDNS Kameras updated
                    Webcam.resetDriver();
                    Webcam.setDriver(WebcamWatchdog.getDriver());
                    Webcam.getDefault();
                    // IpCams hinzufgen
                    SettingsController.this.updateCamList();
                }
            });
        }
        return this.refreshBtn;
    }

    /**
     * Speichert die aktuellen Einstellungen im Settings Objekt.
     */
    public void speichern() {
        // Check DroidCam IP if enabled
        final String ipCam = (String) this.ipCam.getValue();
        final String ipCamIp = (String) this.ipCamHost.getValue();
        if (!Settings.NO_IP_CAM.equals(ipCam) && !Validators.isValidInet4Address(ipCamIp)) { // NOPMD
            GUI.getStatusBar().setErrorText(I18N.tr("SettingsController.wrongIpCanAddress")); //$NON-NLS-1$
        } else {
            final Konto defaultAccount = (Konto) this.defaultAccount.getValue();
            String accountID = StringUtils.EMPTY;
            try {
                accountID = defaultAccount.getID();
            } catch (final RemoteException e) {
                // Nur Loggen
                Logger.error(I18N.tr("SettingsController.errorReadinAccountId"), e); //$NON-NLS-1$
            }
            Settings.setDefaultAccountId(accountID);
            Settings.setPursposeCode((String) this.purposeCode.getValue());
            Settings.setIpCam((String) this.ipCam.getValue());
            Settings.setIpCamHost((String) this.ipCamHost.getValue());
            Settings.setIpCamPort((int) this.ipCamPort.getValue());
            GUI.getStatusBar().setSuccessText(I18N.tr("SettingsController.settingSaved")); //$NON-NLS-1$
            this.updateCamList();
        }
    }

    private void updateCamList() {
        WebcamWatchdog.unregisterIpCams();
        final String camName = (String) this.ipCam.getValue();
        if (!Settings.NO_IP_CAM.equals(camName)) {
            try {
                IpCamDevice ipCam = null;
                switch (camName) {
                    case DroidCamDevice.CAM_NAME:
                        Logger.debug("Adding DroidCam App device: " + Settings.getIpCamHost() + ":" //$NON-NLS-1$ //$NON-NLS-2$
                                + Settings.getIpCamPort());
                        ipCam = new DroidCamDevice(Settings.getIpCamHost(), Settings.getIpCamPort());
                        break;
                    case IphoneIpCameraCamDevice.CAM_NAME:
                        Logger.debug("Adding Iphone IP Camera App device: " + Settings.getIpCamHost() + ":" //$NON-NLS-1$ //$NON-NLS-2$
                                + Settings.getIpCamPort());
                        ipCam = new IphoneIpCameraCamDevice(Settings.getIpCamHost(), Settings.getIpCamPort());
                        break;
                    default:
                        Logger.debug("Adding standard IP cam: " + Settings.getIpCamHost() +  //$NON-NLS-1$
                                ":" + Settings.getIpCamPort()); //$NON-NLS-1$
                        final StringBuffer urlBuilder = new StringBuffer("http://").append(Settings.getIpCamHost()) //$NON-NLS-1$
                                .append(':').append(Settings.getIpCamPort());
                        ipCam = new IpCamDevice(Settings.STANDRD_IP_CAM, urlBuilder.toString(), IpCamMode.PUSH);
                        break;
                }
                if (ipCam != null) {
                    IpCamDeviceRegistry.register(ipCam);
                }
            } catch (final MalformedURLException e) {
                // Nur loggen
                Logger.error(I18N.tr("SettingsController.wrongIpFormat"), e); //$NON-NLS-1$
                GUI.getStatusBar().setErrorText(I18N.tr("SettingsController.wrongIpFormat")); //$NON-NLS-1$
            }
        }
        final StringBuffer webcamNames = new StringBuffer();
        final List<String> dev = WebcamWatchdog.getInstance().getWebcamNames();
        for (final String webcamDevice : dev) {
            Logger.debug("Dev: " + webcamDevice); //$NON-NLS-1$
            webcamNames.append(webcamDevice).append('\n');
        }
        this.discoverdWebcams.setValue(webcamNames.toString());
        this.discoverdWebcams.getControl().update();
    }

    /**
     * @param event
     */
    protected void handleUseDroidCamEvent(final Event event) {
        Logger.debug("ipCam Event" + event.toString()); //$NON-NLS-1$
        if ((boolean) this.ipCam.getValue()) {
            Logger.debug("Enable droidCam Fields"); //$NON-NLS-1$
            this.ipCamHost.enable();
            this.ipCamPort.enable();
        } else {
            Logger.debug("Disable droidCam Fields"); //$NON-NLS-1$
            this.ipCamHost.disable();
            this.ipCamPort.disable();
        }
    }
}
