/*******************************************************************************
 * Copyright 2015 - 2018 fmCreation
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/
package de.frankmuenster.jameica.qrscanner.gui.dialog.threads;

import java.awt.Rectangle;
import java.awt.image.BufferedImage;
import java.util.EnumMap;
import java.util.Map;

import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.widgets.Display;

import com.github.sarxos.webcam.Webcam;
import com.google.zxing.BinaryBitmap;
import com.google.zxing.ChecksumException;
import com.google.zxing.DecodeHintType;
import com.google.zxing.FormatException;
import com.google.zxing.LuminanceSource;
import com.google.zxing.NotFoundException;
import com.google.zxing.Result;
import com.google.zxing.common.HybridBinarizer;
import com.google.zxing.qrcode.QRCodeReader;

import de.frankmuenster.jameica.qrscanner.utils.BufferedImageLuminanceSource;
import de.frankmuenster.jameica.qrscanner.utils.GraphicsUtils;
import de.willuhn.logging.Logger;

/**
 * Liest ein Bild von der angegeben Webcam, bereitet es schn auf und untersucht es auf vorhandene QR-Codes.
 *
 * @author Frank Mnster
 */
public class ScanQrImage {

    /**
     * Die Webcam die benutzt werden soll.
     */
    private final transient Webcam webcam;

    /**
     * Die Gre des Bildes
     */
    private final transient Point size;

    /**
     * Farbe Wei
     */
    private transient Color white;

    /**
     * Farbe dunkel Grau
     */
    private transient Color darkGray;

    /**
     * Farbe grn
     */
    private transient Color green;

    /**
     * Farbe dunkel Grn
     */
    private transient Color darkGreen;

    /**
     * Position der Scanlinie
     */
    private transient int linePosition;

    /**
     * Gre und Position des Scan-Rechtecks
     */
    private transient Rectangle scanRect;

    /**
     * Das letzte gescannte Bild
     */
    private transient BufferedImage bImg;

    /**
     * Erstellt einen ScanQrImage Object. Bilder werden an die angegeben Gre angepasst.
     *
     * @param webcam
     *            Webcam von der gelesen werden soll. Muss bereits geffnet sein.
     * @param size
     *            Die Gre der Bilder die zurck gegeben werden.
     */
    public ScanQrImage(final Webcam webcam, final Point size) {

        this.webcam = webcam;
        this.size = size;
        this.init();
    }

    /**
     * Liest ein Bild von der Webcam, malt ein sch&ouml;nen Rahmen und eine Scan-Linie und gibt das Bild zur&uuml;ck.
     *
     * @return Das Webcam-Bild mit Rahmen und Scan-Linie.
     */
    public Image getImage() {

        this.bImg = this.getImageFromWebcam(this.webcam);
        return this.getImageForDisplay(this.bImg);
    }

    /**
     * Sucht nach einem QR-Code in dem Image
     *
     * @return <b>String</b> mit dem gefundenen QR-Code oder <b>null</b>
     */
    public String getQrDecode() {

        String finalResult = null;
        if (this.bImg == null) {
            this.getImage();
        }
        final LuminanceSource source = new BufferedImageLuminanceSource(this.bImg);
        final BinaryBitmap bitmap = new BinaryBitmap(new HybridBinarizer(source));
        final QRCodeReader qrReader = new QRCodeReader();
        Result result;
        Map<DecodeHintType, Object> tmpHintsMap;
        tmpHintsMap = new EnumMap<>(DecodeHintType.class);
        tmpHintsMap.put(DecodeHintType.TRY_HARDER, Boolean.TRUE);
        try {
            result = qrReader.decode(bitmap, tmpHintsMap);
            // setting results.
            finalResult = String.valueOf(result.getText());
        } catch (final NotFoundException e) {
            // hier brauchen wir nichts zu tun
            Logger.trace("Kein QrCode gefunden."); //$NON-NLS-1$
        } catch (FormatException | ChecksumException e) {
            Logger.trace("Decoder Exeption: " + e.getMessage()); //$NON-NLS-1$
        }
        return finalResult;
    }

    /**
     * Ein paar initiale Berechnungen fr den Rahmen und die Linie.
     */
    private void init() {

        // Grafikdaten vorbereiten
        this.scanRect = new Rectangle();
        // Zentriere scanRechteck
        if (this.size.x < 260 || this.size.y < 240) {
            // Spezialfall
            if (this.size.x < this.size.y) {
                this.scanRect.x = (int) (this.size.x * 0.1);
                this.scanRect.width = (int) (this.size.x * 0.8);
                this.scanRect.y = this.scanRect.x;
                this.scanRect.height = this.scanRect.y;
            } else {
                this.scanRect.x = (int) (this.size.y * 0.1);
                this.scanRect.width = (int) (this.size.y * 0.8);
                this.scanRect.y = this.scanRect.x;
                this.scanRect.height = this.scanRect.y;
            }
        } else {
            this.scanRect.width = 180;
            this.scanRect.height = 180;
            this.scanRect.x = this.size.x / 2 - this.scanRect.width / 2;
            this.scanRect.y = this.size.y / 2 - this.scanRect.height / 2;
        }
        de.willuhn.jameica.gui.GUI.getDisplay().asyncExec(new Runnable() {

            /**
             * Setzten der Farben im asyncExec, da sonst SWT Zugriff Fehler.
             */
            @Override
            public void run() {

                final Display display = de.willuhn.jameica.gui.GUI.getDisplay();
                ScanQrImage.this.green = display.getSystemColor(SWT.COLOR_GREEN);
                ScanQrImage.this.darkGreen = display.getSystemColor(SWT.COLOR_DARK_GREEN);
                ScanQrImage.this.white = display.getSystemColor(SWT.COLOR_WHITE);
                ScanQrImage.this.darkGray = display.getSystemColor(SWT.COLOR_DARK_GRAY);
            }
        });
    }

    /**
     * Liest ein Bild von der angegeben Webcam
     *
     * @param webcam
     *            webcam von der das Bild gelesen werden soll.
     * @return Image als <b>BufferedImage</b>
     */
    private BufferedImage getImageFromWebcam(final Webcam webcam) {

        BufferedImage bImg = null;
        bImg = webcam.getImage();
        bImg = GraphicsUtils.resize(bImg, this.size.x, this.size.y);
        return bImg;
    }

    /**
     * Bereitet das Image fr die Anzeige auf. Es wird ein schner Rahmen und eine Scan-Linie hinzugefgt.
     *
     * @param bImg
     *            image das angezeigt werden soll.
     * @return Image mit Rahmen und Scan-Linie
     */
    private Image getImageForDisplay(final BufferedImage bImg) {

        final Display display = de.willuhn.jameica.gui.GUI.getDisplay();
        final Image img = new Image(display, GraphicsUtils.convertToSWT(bImg));
        // add nice rectangle
        final GC graphicsContext = new GC(img);
        this.drawEdges(graphicsContext);
        this.drawScanLine(graphicsContext);
        graphicsContext.dispose();
        return img;
    }

    /**
     * Malt eine (Scanner)-Linie in 2er Schritten auf ein gc-Objekt
     *
     * @param graphicsContext
     *            gc mit dem Bild auf das die Linie soll
     */
    private void drawScanLine(final GC graphicsContext) {

        final int linePosY = this.scanRect.y - 5 + this.linePosition * 2;
        graphicsContext.setLineStyle(SWT.LINE_SOLID);
        graphicsContext.setLineWidth(3);
        graphicsContext.setForeground(this.darkGreen);
        graphicsContext.drawLine(this.scanRect.x - 10, linePosY, this.scanRect.x + this.scanRect.width + 10, linePosY);
        graphicsContext.setLineWidth(1);
        graphicsContext.setForeground(this.green);
        graphicsContext.drawLine(this.scanRect.x - 15, linePosY, this.scanRect.x + this.scanRect.width + 15, linePosY);
        if (linePosY > this.scanRect.y + this.scanRect.height + 5) {
            this.linePosition = 0;
        } else {
            this.linePosition++;
        }
    }

    /**
     * Malt eine (Scanner)-Rechteck auf ein gc-Objekt
     *
     * @param graphicsContext
     *            gc mit dem Bild auf das die Linie soll
     */
    private void drawEdges(final GC graphicsContext) {

        graphicsContext.setLineStyle(SWT.LINE_SOLID);
        graphicsContext.setLineWidth(1);
        // links-oben
        int pointX = this.scanRect.x;
        int pointY = this.scanRect.y;
        // right
        graphicsContext.setForeground(this.darkGray);
        graphicsContext.drawLine(pointX - 2, pointY - 2, pointX + 17, pointY - 2);
        graphicsContext.drawLine(pointX + 2, pointY + 2, pointX + 13, pointY + 2);
        graphicsContext.setForeground(this.white);
        graphicsContext.drawLine(pointX - 1, pointY - 1, pointX + 16, pointY - 1);
        graphicsContext.drawLine(pointX, pointY, pointX + 15, pointY);
        graphicsContext.drawLine(pointX + 1, pointY + 1, pointX + 14, pointY + 1);
        // down
        graphicsContext.setForeground(this.darkGray);
        graphicsContext.drawLine(pointX - 2, pointY - 2, pointX - 2, pointY + 17);
        graphicsContext.drawLine(pointX + 2, pointY + 2, pointX + 2, pointY + 13);
        graphicsContext.setForeground(this.white);
        graphicsContext.drawLine(pointX - 1, pointY - 1, pointX - 1, pointY + 16);
        graphicsContext.drawLine(pointX, pointY, pointX, pointY + 15);
        graphicsContext.drawLine(pointX + 1, pointY + 1, pointX + 1, pointY + 14);
        // rechts-oben
        pointX = this.scanRect.x + this.scanRect.width;
        pointY = this.scanRect.y;
        // left
        graphicsContext.setForeground(this.darkGray);
        graphicsContext.drawLine(pointX + 2, pointY - 2, pointX - 17, pointY - 2);
        graphicsContext.drawLine(pointX - 2, pointY + 2, pointX - 13, pointY + 2);
        graphicsContext.setForeground(this.white);
        graphicsContext.drawLine(pointX + 1, pointY - 1, pointX - 16, pointY - 1);
        graphicsContext.drawLine(pointX, pointY, pointX - 15, pointY);
        graphicsContext.drawLine(pointX - 1, pointY + 1, pointX - 14, pointY + 1);
        // down
        graphicsContext.setForeground(this.darkGray);
        graphicsContext.drawLine(pointX + 2, pointY - 2, pointX + 2, pointY + 17);
        graphicsContext.drawLine(pointX - 2, pointY + 2, pointX - 2, pointY + 13);
        graphicsContext.setForeground(this.white);
        graphicsContext.drawLine(pointX + 1, pointY - 1, pointX + 1, pointY + 16);
        graphicsContext.drawLine(pointX, pointY, pointX, pointY + 15);
        graphicsContext.drawLine(pointX - 1, pointY + 1, pointX - 1, pointY + 14);
        // links-unten
        pointX = this.scanRect.x;
        pointY = this.scanRect.y + this.scanRect.height;
        // right
        graphicsContext.setForeground(this.darkGray);
        graphicsContext.drawLine(pointX + 2, pointY - 2, pointX + 13, pointY - 2);
        graphicsContext.drawLine(pointX - 2, pointY + 2, pointX + 17, pointY + 2);
        graphicsContext.setForeground(this.white);
        graphicsContext.drawLine(pointX + 1, pointY - 1, pointX + 14, pointY - 1);
        graphicsContext.drawLine(pointX, pointY, pointX + 15, pointY);
        graphicsContext.drawLine(pointX - 1, pointY + 1, pointX + 16, pointY + 1);
        // up
        graphicsContext.setForeground(this.darkGray);
        graphicsContext.drawLine(pointX - 2, pointY + 2, pointX - 2, pointY - 17);
        graphicsContext.drawLine(pointX + 2, pointY - 2, pointX + 2, pointY - 13);
        graphicsContext.setForeground(this.white);
        graphicsContext.drawLine(pointX - 1, pointY + 1, pointX - 1, pointY - 16);
        graphicsContext.drawLine(pointX, pointY, pointX, pointY - 15);
        graphicsContext.drawLine(pointX + 1, pointY - 1, pointX + 1, pointY - 14);
        // rechts-unten
        pointX = this.scanRect.x + this.scanRect.width;
        pointY = this.scanRect.y + this.scanRect.height;
        // left
        graphicsContext.setForeground(this.darkGray);
        graphicsContext.drawLine(pointX - 2, pointY - 2, pointX - 13, pointY - 2);
        graphicsContext.drawLine(pointX + 2, pointY + 2, pointX - 17, pointY + 2);
        graphicsContext.setForeground(this.white);
        graphicsContext.drawLine(pointX - 1, pointY - 1, pointX - 14, pointY - 1);
        graphicsContext.drawLine(pointX, pointY, pointX - 15, pointY);
        graphicsContext.drawLine(pointX + 1, pointY + 1, pointX - 16, pointY + 1);
        // up
        graphicsContext.setForeground(this.darkGray);
        graphicsContext.drawLine(pointX - 2, pointY - 2, pointX - 2, pointY - 13);
        graphicsContext.drawLine(pointX + 2, pointY + 2, pointX + 2, pointY - 17);
        graphicsContext.setForeground(this.white);
        graphicsContext.drawLine(pointX - 1, pointY - 1, pointX - 1, pointY - 14);
        graphicsContext.drawLine(pointX, pointY, pointX, pointY - 15);
        graphicsContext.drawLine(pointX + 1, pointY + 1, pointX + 1, pointY - 16);
    }
}
